using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace KeyExaminer
{
    public partial class Form1 : Form
    {
        int[][] keys;
        double[][] times;

        double [,,] timeSum;
        double[,,] time2Sum;
        int[,,] timeNum;

        string[] files = { @"..\..\..\jl.cvs", @"..\..\..\jl_new.cvs", @"..\..\..\jl_twofingers.cvs"};

        int fileDisplayed;
        bool showParallelCoords;

        public Form1()
        {
            InitializeComponent();

            keys = new int[files.Length][];
            times = new double[files.Length][];
            timeSum = new double[files.Length, 64, 64];
            time2Sum = new double[files.Length, 64, 64];
            timeNum = new int[files.Length, 64, 64];

            for (int f = 0; f < files.Length; f++)
            {
                string[] data = File.ReadAllLines(files[f]);
                keys[f] = new int[data.Length];
                times[f] = new double[data.Length];
                for (int i = 0; i < data.Length; i++)
                {
                    string[] s = data[i].Split(';');
                    keys[f][i] = Int32.Parse(s[0]);
                    times[f][i] = Double.Parse(s[1]);
                }
                for (int i = 0; i < data.Length - 1; i++)
                {
                    int x = keys[f][i] - 64;
                    if (x < 0 || x >= 64) continue;
                    int y = keys[f][i + 1] - 64;
                    if (y < 0 || y >= 64) continue;

                    double timeDiff = times[f][i + 1] - times[f][i];
                    timeSum[f, x, y] += timeDiff;
                    time2Sum[f, x, y] += timeDiff * timeDiff;
                    timeNum[f, x, y]++;
                }
            }
        }

        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            base.OnKeyPress(e);

            if (e.KeyChar == 'f')
            {
                fileDisplayed++;
                if(fileDisplayed >= files.Length)
                {
                    fileDisplayed = 0;
                }
                Text = fileDisplayed.ToString();
                Invalidate();
            }
            else if (e.KeyChar == 'p')
            {
                showParallelCoords = !showParallelCoords;
                Invalidate();
            }
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            for (int x = 0; x < 64; x++)
            {
                for (int y = 0; y < 64; y++)
                {
                    Rectangle r = new Rectangle(x * 8, y * 8, 8,8);
                    if(r.Contains(e.Location))
                    {
                        Text = (char)(x+64) + "->" + (char)(y+64);
                    }
                }
            }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            
            base.OnPaint(e);

            if (showParallelCoords)
            {
                for (int f = 0; f < files.Length; f++)
                {
                    double x1 = timeSum[f, 'a' - 64, 'n' - 64] / timeNum[f, 'a' - 64, 'n' - 64];
                    double x3 = timeSum[f, 'e' - 64, 'n' - 64] / timeNum[f, 'e' - 64, 'n' - 64];
                    double x5 = timeSum[f, 'i' - 64, 'n' - 64] / timeNum[f, 'i' - 64, 'n' - 64];
                    double x2 = time2Sum[f, 'a' - 64, 'n' - 64] / timeNum[f, 'a' - 64, 'n' - 64] - timeSum[f, 'a' - 64, 'n' - 64] / timeNum[f, 'a' - 64, 'n' - 64] * timeSum[f, 'a' - 64, 'n' - 64] / timeNum[f, 'a' - 64, 'n' - 64];
                    double x4 = time2Sum[f, 'e' - 64, 'n' - 64] / timeNum[f, 'e' - 64, 'n' - 64] - timeSum[f, 'e' - 64, 'n' - 64] / timeNum[f, 'e' - 64, 'n' - 64] * timeSum[f, 'e' - 64, 'n' - 64] / timeNum[f, 'e' - 64, 'n' - 64];
                    double x6 = time2Sum[f, 'i' - 64, 'n' - 64] / timeNum[f, 'i' - 64, 'n' - 64] - timeSum[f, 'i' - 64, 'n' - 64] / timeNum[f, 'i' - 64, 'n' - 64] * timeSum[f, 'i' - 64, 'n' - 64] / timeNum[f, 'i' - 64, 'n' - 64];

                    Pen p = (f == fileDisplayed) ? Pens.Red : Pens.Black;

                    e.Graphics.DrawLine(p, 20, 200 - (float)(100.0 * x1 / 0.2), 40, 200 - (float)(100.0 * x2 / 0.2));
                    e.Graphics.DrawLine(p, 40, 200 - (float)(100.0 * x2 / 0.2), 60, 200 - (float)(100.0 * x3 / 0.2));
                    e.Graphics.DrawLine(p, 60, 200 - (float)(100.0 * x3 / 0.2), 80, 200 - (float)(100.0 * x4 / 0.2));
                    e.Graphics.DrawLine(p, 80, 200 - (float)(100.0 * x4 / 0.2), 100, 200 - (float)(100.0 * x5 / 0.2));
                    e.Graphics.DrawLine(p, 100, 200 - (float)(100.0 * x5 / 0.2), 120, 200 - (float)(100.0 * x6 / 0.2));
                }
            }
            else
            {
                for (int x = 0; x < 64; x++)
                {
                    for (int y = 0; y < 64; y++)
                    {
                        Color c = Color.Blue;
                        if (timeNum[fileDisplayed, x, y] > 0)
                        {
                            int r = (int)(255.0 / 0.2 * timeSum[fileDisplayed, x, y] / timeNum[fileDisplayed, x, y]);
                            int g = (int)(255.0 / 0.1 * Math.Sqrt(time2Sum[fileDisplayed, x, y] / timeNum[fileDisplayed, x, y] - (timeSum[fileDisplayed, x, y] / timeNum[fileDisplayed, x, y]) * (timeSum[fileDisplayed, x, y] / timeNum[fileDisplayed, x, y])));
                            r = Math.Min(255, Math.Max(0, r));
                            g = Math.Min(255, Math.Max(0, g));
                            c = Color.FromArgb(r, g, 0);
                        }
                        e.Graphics.FillRectangle(new SolidBrush(c), x * 8, y * 8, 8, 8);
                        if (timeNum[fileDisplayed, x, y] > keys[fileDisplayed].Length / 100)
                        {
                            e.Graphics.DrawRectangle(Pens.White, x * 8, y * 8, 7, 7);
                        }
                    }
                }
            }
        }
    }
}